module SenPlanck_GK2A

!====================================================================
!
! Module:
!      SenPlanck_GK2A
!
! Object:
!      This module contains subroutines to convert the sensor
!      radiant quntities of the GK2A infrared channels
!      between radiance and brightness temeprature.
!
! Functions:
!      Tb -> I(wavenumber) :      SenPlanck_temp2rad_wavenum_[quad or line]()
!      Tb -> I(wavelength) :      SenPlanck_temp2rad_wavelen_[quad or line]()
!      I(wavenumber) -> Tb :      SenPlanck_rad2temp_wavenum_[quad or line]()
!      I(wavelength) -> Tb :      SenPlanck_rad2temp_wavelen_[quad or line]()
!
!      1) SenPlanck_*_quad estimates effective temperature by 
!         quadratic approximation, where SenPlanck_*_line does
!         by linear approximation.
!      2) SenPlanck_*_quad supports the TB range from 130 K to 330 K.
!         SenPlanck_*_line supports the TB range from 130 K to 330 K. (updatedy by Dohy)
!
! Notice:
!      1) The central frequencies and the band conversion parameters were
!         computed by Dohy in March 2019.
!
! Created by Y.Tahara in Nov 2006
! GK2A/AMI parameters modified by Dohy in 19 March 2019
!
!====================================================================

  !--- implicit

  implicit none

  !--- public & private

  private

  public  SenPlanck_rad2temp_wavenum_line
  public  SenPlanck_rad2temp_wavelen_line
  public  SenPlanck_temp2rad_wavenum_line
  public  SenPlanck_temp2rad_wavelen_line

  public  SenPlanck_rad2temp_wavenum_quad
  public  SenPlanck_rad2temp_wavelen_quad
  public  SenPlanck_temp2rad_wavenum_quad
  public  SenPlanck_temp2rad_wavelen_quad

  !--- parameters
  
  integer,         parameter :: fp_kind = SELECTED_REAL_KIND( 15 )      ! Double precision

  real( fp_kind ), parameter :: ONE = 1.0_fp_kind

  !--- sensor spectral variables

  integer, parameter    :: n_Channels = 10            ! # of channels

  type, public :: SpcCoeff_Def
    !-- sensor and channel name
    character( len=80 ) :: senchan
    !-- central spectrum
    real( fp_kind )     :: spectrum                               ! central spectrum
    !-- Planck sensor coeffients
    real( fp_kind )     :: planck_c1                              ! Planck sensor coeffients (C1)
    real( fp_kind )     :: planck_c2                              !                                             (C2)
    !-- band correction coefficients (T --> Teff, linear approximation)
    real( fp_kind )     :: t2e_d1                                    ! Teff = a1 + a2*T
    real( fp_kind )     :: t2e_d2                                  
    !-- band correction coefficients (T --> Teff, quadratic approximation)
    real( fp_kind )     :: t2e_c1                                     ! Teff = b1 + b2*T + b3*T^2
    real( fp_kind )     :: t2e_c2                                  
    real( fp_kind )     :: t2e_c3
    !-- band correction coefficients (Teff --> T, quadratic approximation)
    real( fp_kind )     :: e2t_c1                                     ! T = c1 + c2*Teff + c3*Teff^2
    real( fp_kind )     :: e2t_c2
    real( fp_kind )     :: e2t_c3
  end type SpcCoeff_Def

  type( SpcCoeff_Def ), public, save :: SpcCoeff_wavelen( n_Channels )      ! Spectral coefficients
  type( SpcCoeff_Def ), public, save :: SpcCoeff_wavenum( n_Channels )

  logical, save       :: Flag_wavelen = .false.            ! Flags of initializetion
  logical, save       :: Flag_wavenum = .false.

 contains

  !====================================================================
  !
  ! Function:
  !      SenPlanck_temp2rad_wavenum_quad( channel, temperature )
  !      SenPlanck_temp2rad_wavenum_line( channel, temperature )
  !      result( radiance )
  !
  ! Object:
  !      To compute a sensor radiance in unit of wavenumber from
  !      a brightness temperature.
  !      SenPlanck_*_quad estimates effective temperature by
  !      quadratic approximation, where SenPlanck_*_line does
  !      by linear approximation.
  !
  ! Interface:
  !      in)  channel            seq # of a channel
  !                        unit: none
  !                        type: integer
  !                    
  !      in)  temperature      brightness temperature
  !                        unit: K
  !                        type: double
  !
  !      out) radiance            sensor radiance of the channel
  !                           unit: mW/(m^2.sr.cm^-1)
  !                        type: double
  !
  ! Equation:
  !               2 h c^2 wn^3                    C1
  !   B = ---------------------------- = ----------------------
  !        exp( h c wn / K Teff ) - 1     exp( C2 / Teff ) - 1
  !
  !   (linear) Teff = a1 + a2 * T
  !   (quadr.) Teff = b1 + b2 * T + b3 * T^2
  !
  !      h  : Planck's constant         C1 = 2 h c^2 wn^3
  !      K  : Boltzmann's constant      C2 = h c wn / K
  !      c  : speed of light
  !      wn : central wavenumber
  !      T  : brightness temperature      Teff : effective temperature
  !      ai, bi : band correction coefficients
  !
  !====================================================================

  function SenPlanck_temp2rad_wavenum_quad( channel, temperature ) result( radiance )

    !--- interface

    integer        , intent( in ) :: channel    
    real( fp_kind ), intent( in ) :: temperature
    real( fp_kind )               :: radiance   

    !--- local variables

    real(fp_kind) :: effective_temperature

    !--- initialization
    
    if( .not. Flag_wavenum )then
      call Init_SpcCoef_wavenum()
    endif
!print*, channel, temperature

    !--- compute effective temperature used for
    !--- the correction of sensor polychromaticity

    effective_temperature =   SpcCoeff_wavenum( channel )%t2e_c1              &
                            + SpcCoeff_wavenum( channel )%t2e_c2 * temperature    &
                            + SpcCoeff_wavenum( channel )%t2e_c3 * temperature**2
!print*,effective_temperature

    !--- compute sensor radiance

    radiance =                  SpcCoeff_wavenum( channel )%planck_c1  / &
    !          ------------------------------------------------------------------------------
               ( EXP( SpcCoeff_wavenum( channel )%planck_c2 / effective_temperature ) - ONE )
!print*,radiance

  end function SenPlanck_temp2rad_wavenum_quad



  function SenPlanck_temp2rad_wavenum_line( channel, temperature ) result( radiance )

    !--- interface

    integer        , intent( in ) :: channel    
    real( fp_kind ), intent( in ) :: temperature
    real( fp_kind )               :: radiance   

    !--- local variables

    real(fp_kind) :: effective_temperature

    !--- initialization
    
    if( .not. Flag_wavenum )then
      call Init_SpcCoef_wavenum()
    endif

    !--- compute effective temperature used for
    !--- the correction of sensor polychromaticity

    effective_temperature =   SpcCoeff_wavenum( channel )%t2e_d1              &
                            + SpcCoeff_wavenum( channel )%t2e_d2 * temperature

    !--- compute sensor radiance

    radiance =                  SpcCoeff_wavenum( channel )%planck_c1  / &
    !          ------------------------------------------------------------------------------
               ( EXP( SpcCoeff_wavenum( channel )%planck_c2 / effective_temperature ) - ONE )

  end function SenPlanck_temp2rad_wavenum_line

  !====================================================================
  !
  ! Function:
  !      SenPlanck_rad2temp_wavenum_quad( channel, radiance )
  !      SenPlanck_rad2temp_wavenum_line( channel, radiance )
  !      result( temperature )
  !
  ! Object:
  !      To compute a brightness temperature from a sensor radiance
  !      in unit of wavenumber.
  !      SenPlanck_*_quad estimates effective temperature by
  !      quadratic approximation, where SenPlanck_*_line does
  !      by linear approximation.
  !
  ! Interface:
  !      in)  channel            seq # of a channel
  !                        unit: none
  !                        type: integer
  !                    
  !      in) radiance            sensor radiance of the channel
  !                           unit: mW/(m^2.sr.cm^-1)
  !                        type: double
  !                    
  !      out)  temperature      brightness temperature
  !                        unit: K
  !                        type: double
  !
  ! Equation:
  !               2 h c^2 wn^3                    C1
  !   B = ---------------------------- = ----------------------
  !        exp( h c wn / K Teff ) - 1     exp( C2 / Teff ) - 1
  !
  !   (linear) T = ( Teff - a1 ) / a2
  !   (quadr.) T = c1 + c2 * Teff + c3 * Teff^2
  !
  !      h  : Planck's constant         C1 = 2 h c^2 wn^3
  !      K  : Boltzmann's constant      C2 = h c wn / K
  !      c  : speed of light
  !      wn : central wavenumber
  !      T  : brightness temperature      Teff : effective temperature
  !      ai, ci : band correction coefficients
  !
  !====================================================================

  function SenPlanck_rad2temp_wavenum_quad( channel, radiance ) result( temperature )

    !--- interface

    integer        , intent( in ) :: channel     
    real( fp_kind ), intent( in ) :: radiance    
    real( fp_kind )               :: temperature 

    !--- local variables

    real(fp_kind) :: effective_temperature

    !--- initialization
    
    if( .not. Flag_wavenum )then
      call Init_SpcCoef_wavenum()
    endif

    !--- compute effective temperature

    effective_temperature =              SpcCoeff_wavenum( channel )%planck_c2  / &
    !                       -----------------------------------------------------------------
                            LOG( ( SpcCoeff_wavenum( channel )%planck_c1 / radiance ) + ONE )

    !--- convert the effective temperature to brightness temperature

    temperature =   SpcCoeff_wavenum( channel )%e2t_c1                      &
                  + SpcCoeff_wavenum( channel )%e2t_c2 * effective_temperature    &
                  + SpcCoeff_wavenum( channel )%e2t_c3 * effective_temperature**2

  end function SenPlanck_rad2temp_wavenum_quad



  function SenPlanck_rad2temp_wavenum_line( channel, radiance ) result( temperature )

    !--- interface

    integer        , intent( in ) :: channel     
    real( fp_kind ), intent( in ) :: radiance    
    real( fp_kind )               :: temperature 

    !--- local variables

    real(fp_kind) :: effective_temperature

    !--- initialization
    
    if( .not. Flag_wavenum )then
      call Init_SpcCoef_wavenum()
    endif

    !--- compute effective temperature

    effective_temperature =              SpcCoeff_wavenum( channel )%planck_c2  / &
    !                       -----------------------------------------------------------------
                            LOG( ( SpcCoeff_wavenum( channel )%planck_c1 / radiance ) + ONE )

    !--- convert the effective temperature to brightness temperature

    temperature = ( effective_temperature - SpcCoeff_wavenum( channel )%t2e_d1 ) / &
    !             ------------------------------------------------------------------
                                            SpcCoeff_wavenum( channel )%t2e_d2

  end function SenPlanck_rad2temp_wavenum_line

  !====================================================================
  !
  ! Function:
  !      SenPlanck_temp2rad_wavelen_quad( channel, temperature )
  !      SenPlanck_temp2rad_wavelen_line( channel, temperature )
  !      result( radiance )
  !
  ! Object:
  !      To compute a sensor radiance in unit of wavelength from
  !      a brightness temperature.
  !      SenPlanck_*_quad estimates effective temperature by
  !      quadratic approximation, where SenPlanck_*_line does
  !      by linear approximation.
  !
  ! Interface:
  !      in)  channel            seq # of a channel
  !                        unit: none
  !                        type: integer
  !                    
  !      in)  temperature      brightness temperature
  !                        unit: K
  !                        type: double
  !
  !      out) radiance            sensor radiance of the channel
  !                           unit: W/(m^2.sr.micron)
  !                        type: double
  !
  ! Equation:
  !                   2 h c^2                             C1
  !   B = ------------------------------------- = ----------------------
  !        lm^5 ( exp( h c / K Teff lm ) - 1 )     exp( C2 / Teff ) - 1
  !
  !   (linear) Teff = a1 + a2 * T
  !   (quadr.) Teff = b1 + b2 * T + b3 * T^2
  !
  !      h  : Planck's constant         C1 = 2 h c^2 / lm^5
  !      K  : Boltzmann's constant      C2 = h c / K lm
  !      c  : speed of light
  !      lm : central wavelength
  !      T  : brightness temperature      Teff : effective temperature
  !      ai,bi : band correction coefficients
  !
  !====================================================================

  function SenPlanck_temp2rad_wavelen_quad( channel, temperature ) result( radiance )

    !--- interface

    integer        , intent( in ) :: channel    
    real( fp_kind ), intent( in ) :: temperature
    real( fp_kind )               :: radiance   

    !--- local variables

    real(fp_kind) :: effective_temperature

    !--- initialization
    
    if( .not. Flag_wavelen )then
      call Init_SpcCoef_wavelen()
    endif

    !--- compute effective temperature used for
    !--- the correction of sensor polychromaticity

    effective_temperature =   SpcCoeff_wavelen( channel )%t2e_c1              &
                            + SpcCoeff_wavelen( channel )%t2e_c2 * temperature    &
                            + SpcCoeff_wavelen( channel )%t2e_c3 * temperature**2

    !--- compute sensor radiance

    radiance =                  SpcCoeff_wavelen( channel )%planck_c1  / &
    !          ------------------------------------------------------------------------------
               ( EXP( SpcCoeff_wavelen( channel )%planck_c2 / effective_temperature ) - ONE )

  end function SenPlanck_temp2rad_wavelen_quad



  function SenPlanck_temp2rad_wavelen_line( channel, temperature ) result( radiance )

    !--- interface

    integer        , intent( in ) :: channel    
    real( fp_kind ), intent( in ) :: temperature
    real( fp_kind )               :: radiance   

    !--- local variables

    real(fp_kind) :: effective_temperature

    !--- initialization
    
    if( .not. Flag_wavelen )then
      call Init_SpcCoef_wavelen()
    endif

    !--- compute effective temperature used for
    !--- the correction of sensor polychromaticity

    effective_temperature =   SpcCoeff_wavelen( channel )%t2e_d1              &
                            + SpcCoeff_wavelen( channel )%t2e_d2 * temperature

    !--- compute sensor radiance

    radiance =                  SpcCoeff_wavelen( channel )%planck_c1  / &
    !          ------------------------------------------------------------------------------
               ( EXP( SpcCoeff_wavelen( channel )%planck_c2 / effective_temperature ) - ONE )

  end function SenPlanck_temp2rad_wavelen_line



  !====================================================================
  !
  ! Function:
  !      SenPlanck_rad2temp_wavelen_quad( channel, radiance )
  !      SenPlanck_rad2temp_wavelen_line( channel, radiance )
  !      result( temperature )
  !
  ! Object:
  !      To compute a brightness temperature from a sensor radiance
  !      in unit of wavelength.
  !      SenPlanck_*_quad estimates effective temperature by
  !      quadratic approximation, where SenPlanck_*_line does
  !      by linear approximation.
  !
  ! Interface:
  !      in)  channel            seq # of a channel
  !                        unit: none
  !                        type: integer
  !
  !      in)  radiance            sensor radiance of the channel
  !                           unit: W/(m^2.sr.micron)
  !                        type: double
  !                    
  !      out) temperature      brightness temperature
  !                        unit: K
  !                        type: double
  !
  ! Equation:
  !                   2 h c^2                             C1
  !   B = ------------------------------------- = ----------------------
  !        lm^5 ( exp( h c / K Teff lm ) - 1 )     exp( C2 / Teff ) - 1
  !
  !   (linear) T = ( Teff - a1 ) / a2
  !   (quadr.) T = c1 + c2 * Teff + c3 * Teff^2
  !
  !      h  : Planck's constant         C1 = 2 h c^2 wn^3
  !      K  : Boltzmann's constant      C2 = h c wn / K
  !      c  : speed of light
  !      wn : central wavenumber
  !      T  : brightness temperature      Teff : effective temperature
  !      ai, ci : band correction coefficients
  !
  !====================================================================

  function SenPlanck_rad2temp_wavelen_quad( channel, radiance ) result( temperature )

    !--- interface

    integer        , intent( in ) :: channel                ! seq # of a GK2A infrared channel (1-10)
    real( fp_kind ), intent( in ) :: radiance		    ! sensor radiance of the channel ( W/(m^2.sr.micron) )
    real( fp_kind )               :: temperature           ! brightness temperature ( K )

    !--- local variables

    real(fp_kind) :: effective_temperature

    !--- initialization
    
    if( .not. Flag_wavelen )then
      call Init_SpcCoef_wavelen()
    endif

    !--- compute effective temperature

    effective_temperature =              SpcCoeff_wavelen( channel )%planck_c2  / &
    !                       -----------------------------------------------------------------
                            LOG( ( SpcCoeff_wavelen( channel )%planck_c1 / radiance ) + ONE )

    !--- convert the effective temperature to brightness temperature

    temperature =   SpcCoeff_wavelen( channel )%e2t_c1                      &
                  + SpcCoeff_wavelen( channel )%e2t_c2 * effective_temperature    &
                  + SpcCoeff_wavelen( channel )%e2t_c3 * effective_temperature**2

  end function SenPlanck_rad2temp_wavelen_quad



  function SenPlanck_rad2temp_wavelen_line( channel, radiance ) result( temperature )

    !--- interface

    integer        , intent( in ) :: channel                  ! seq # of a GK2A infrared channel (1-10)
    real( fp_kind ), intent( in ) :: radiance            ! sensor radiance of the channel ( W/(m^2.sr.micron) )
    real( fp_kind )               :: temperature            ! brightness temperature ( K )

    !--- local variables

    real(fp_kind) :: effective_temperature

    !--- initialization
    
    if( .not. Flag_wavelen )then
      call Init_SpcCoef_wavelen()
    endif

    !--- compute effective temperature

    effective_temperature =              SpcCoeff_wavelen( channel )%planck_c2  / &
    !                       -----------------------------------------------------------------
                            LOG( ( SpcCoeff_wavelen( channel )%planck_c1 / radiance ) + ONE )

    !--- convert the effective temperature to brightness temperature

    temperature = ( effective_temperature - SpcCoeff_wavelen( channel )%t2e_d1 ) / &
    !             ------------------------------------------------------------------
                                            SpcCoeff_wavelen( channel )%t2e_d2

  end function SenPlanck_rad2temp_wavelen_line



  !====================================================================
  !
  ! Private subroutines:
  !      Init_SpcCoef_wavenum()
  !      Init_SpcCoef_wavelen()
  !
  ! Object:
  !      To set sensor spectral coefficients to a structural variable.
  !
  !====================================================================


  !-- sensor coefficients for radaince in unit of wavenumber

  subroutine Init_SpcCoef_wavenum()

    integer :: i_channel

    Flag_wavenum = .true.
    i_channel    = 0
 
    !-- GK2A IR1 (ch.7)

    i_channel = i_channel + 1
    SpcCoeff_wavenum( i_channel ) = SpcCoeff_Def( &
         "GK2A IR1", &
       2.612677373521114100E+03_fp_kind,                                                       &
       2.124147606094285700E+05_fp_kind, 3.759054920825371000E+03_fp_kind, &
       4.444158866114768500E-01_fp_kind, 9.993739053425847500E-01_fp_kind,                                     &
       4.475505952256744700E-01_fp_kind, 9.993448041122612300E-01_fp_kind, 6.326354418285791300E-08_fp_kind,            &
      -4.478439398241241300E-01_fp_kind,  1.000655680903896E+00_fp_kind, -6.338240899124487500E-08_fp_kind )

    !-- GK2A IR2 (ch.8)

    i_channel = i_channel + 1
    SpcCoeff_wavenum( i_channel ) = SpcCoeff_Def( &
         "GK2A IR2", &
       1.617609242531345900E+03_fp_kind,      &
       5.041365661080964500E+04_fp_kind, 2.327375758192103600E+03_fp_kind,                                    &
       1.7073478337171852_fp_kind, 9.963185178001206000E-01_fp_kind,                                     &
       1.7555353195531502_fp_kind, 9.958711667711691600E-01_fp_kind,  9.725022368521507900E-07_fp_kind,            &
      -1.7627949401114713_fp_kind,  1.0041491056227825_fp_kind, -9.833109143193855900E-07_fp_kind )

    !-- GK2A IR3 (ch.9)

    i_channel = i_channel + 1
    SpcCoeff_wavenum( i_channel ) = SpcCoeff_Def( &
         "GK2A IR3", &
       1.441575428760172100E+03_fp_kind,      &
       3.568120015540689200E+04_fp_kind, 2.074102705577733600E+03_fp_kind,                                    &
       3.095387343991924900E-01_fp_kind, 9.992546126003693000E-01_fp_kind,                                     &
       3.339915549098138600E-01_fp_kind, 9.990276035704011000E-01_fp_kind, 4.934978912378580700E-07_fp_kind,            &
      -3.343114143591067700E-01_fp_kind,  1.0009735987446888_fp_kind, -4.946030702523042400E-07_fp_kind )

    !-- GK2A IR4 (ch.10)

    i_channel = i_channel + 1
    SpcCoeff_wavenum( i_channel ) = SpcCoeff_Def( &
         "GK2A IR4", &
       1.365249992024443100E+03_fp_kind,      &
       3.030846124376614900E+04_fp_kind, 1.964287574381907200E+03_fp_kind,                                    &
       5.605775541330511300E-02_fp_kind, 9.998586350844365100E-01_fp_kind,                                     &
       6.130107531190725400E-02_fp_kind, 9.998099584516744200E-01_fp_kind,  1.058187668761811100E-07_fp_kind,            &
      -6.131248596965952600E-02_fp_kind,  1.0001900872294154_fp_kind, -1.058636567504995900E-07_fp_kind )

    !-- GK2A IR5(ch.11)

    i_channel = i_channel + 1
    SpcCoeff_wavenum( i_channel ) = SpcCoeff_Def( &
         "GK2A IR5", &
       1.164949392856341500E+03_fp_kind,      &
       1.882992186706090800E+04_fp_kind, 1.676100077304003200E+03_fp_kind,                                    &
       1.233863508941170600E-01_fp_kind, 9.996447496017424300E-01_fp_kind,                                     &
       1.413475405027945800E-01_fp_kind, 9.994780059646083400E-01_fp_kind,  3.624861676846743900E-07_fp_kind,      &
      -1.414185282031553700E-01_fp_kind,  1.0005223290688519_fp_kind, -3.628727607610907900E-07_fp_kind )

    !-- GK2A IR6 (ch.12)

    i_channel = i_channel + 1
    SpcCoeff_wavenum( i_channel ) = SpcCoeff_Def( &
         "GK2A IR6", &
       1.039960216776113400E+03_fp_kind,      &
       1.339607336888839000E+04_fp_kind,  1.496268773923025700E+03_fp_kind,                                    &
       9.540568903432479500E-02_fp_kind, 9.996987737468530100E-01_fp_kind,                                     &
       1.139667621786082400E-01_fp_kind, 9.995264610597418600E-01_fp_kind,  3.745927980698491500E-07_fp_kind,      &
      -1.140177281581982400E-01_fp_kind,  1.0004738058540292_fp_kind, -3.749315099284038100E-07_fp_kind )

    !-- GK2A IR7 (ch.13)

    i_channel = i_channel + 1
    SpcCoeff_wavenum( i_channel ) = SpcCoeff_Def( &
         "GK2A IR7", &
       9.661533839260550800E+02_fp_kind,      &
       1.074151595347733900E+04_fp_kind,  1.390077347064364400E+03_fp_kind,                                    &
       1.155387158379266200E-01_fp_kind, 9.996126890112760900E-01_fp_kind,                                     &
       1.427815026943483100E-01_fp_kind, 9.993597791837491900E-01_fp_kind,  5.498039728855791100E-07_fp_kind,      &
      -1.428664484751778800E-01_fp_kind,  1.0006406957204939_fp_kind, -5.504432949604983900E-07_fp_kind )

    !-- GK2A IR8 (ch.14)
	
    i_channel = i_channel + 1
    SpcCoeff_wavenum( i_channel ) = SpcCoeff_Def( &
         "GK2A IR8", &
        8.917130573012607300E+02_fp_kind,      &
        8.445059581681534800E+03_fp_kind,  1.282974465192018900E+03_fp_kind,                                    &
       1.928791708469707000E-01_fp_kind, 9.993094649958030900E-01_fp_kind,                                     &
        2.488378141829691900E-01_fp_kind, 9.987899700223505900E-01_fp_kind,  1.129336898812135100E-06_fp_kind,      &
      -2.491117184961483300E-01_fp_kind,  1.0012116687375692_fp_kind, -1.131679640116656700E-06_fp_kind )

    !-- GK2A IR9 (ch.15)

    i_channel = i_channel + 1
    SpcCoeff_wavenum( i_channel ) = SpcCoeff_Def( &
         "GK2A IR9", &
       8.106090078712308000E+02_fp_kind,      &
        6.343977242759571700E+03_fp_kind,  1.166283985457073000E+03_fp_kind,                                    &
       3.322203871923244400E-01_fp_kind, 9.987107652252591400E-01_fp_kind,                                     &
       4.571293989378905600E-01_fp_kind, 9.975511659234833600E-01_fp_kind,  2.520868047341300200E-06_fp_kind,      &
      -4.581138857227389200E-01_fp_kind,  1.0024552097553592_fp_kind, -2.530643147204760400E-06_fp_kind )

    !-- GK2A IR10 (ch.16)

    i_channel = i_channel + 1
    SpcCoeff_wavenum( i_channel ) = SpcCoeff_Def( &
         "GK2A IR10", &
       7.535906214822786100E+02_fp_kind,      &
       5.097224885942314400E+03_fp_kind,  1.084247355865852800E+03_fp_kind,                                    &
       6.435469410936545100E-02_fp_kind, 9.997339156484691400E-01_fp_kind,                                     &
       9.380914369146961500E-02_fp_kind, 9.994604737350933900E-01_fp_kind,  5.944389421236717800E-07_fp_kind,      &
      -9.385215685276571100E-02_fp_kind,  1.0005398211296685_fp_kind, -5.949137153128491800E-07_fp_kind )

  end subroutine Init_SpcCoef_wavenum


  !-- sensor coefficients for radaince in unit of wavelength 

  subroutine Init_SpcCoef_wavelen()

    integer :: i_channel

    Flag_wavelen = .true.
    i_channel    = 0

    !-- GK2A IR1 (ch.7)

    i_channel = i_channel + 1
    SpcCoeff_wavelen( i_channel ) = SpcCoeff_Def( &
         "GK2A IR1", &
       3.830302364276442_fp_kind, &
       1.444648021348663600E+05_fp_kind, 3.756296176377843500E+03_fp_kind,                                    &
       4.054455480197520200E-01_fp_kind, 9.989998918018321300E-01_fp_kind,                                          &
       4.472285325004012900E-01_fp_kind, 9.986119972940398200E-01_fp_kind,  8.432489299849614300E-07_fp_kind,      &
      -4.478348543602380100E-01_fp_kind,  1.0013904705653804_fp_kind, -8.457839043150996100E-07_fp_kind )

    !-- GK2A IR2 (ch.8)

    i_channel = i_channel + 1
    SpcCoeff_wavelen( i_channel ) = SpcCoeff_Def( &
         "GK2A IR2", &
       6.2116606776392311_fp_kind, &
       1.287919508508354700E+04_fp_kind, 2.316248564107133500E+03_fp_kind,                                    &
       1.2950275337529566_fp_kind, 9.953335859876062400E-01_fp_kind,                                          &
       1.7468151130570255_fp_kind, 9.911393925167710800E-01_fp_kind, 9.117811893122135400E-06_fp_kind,      &
      -1.7606059055286278_fp_kind,  1.0089462972844623_fp_kind, -9.246638035855573300E-06_fp_kind )

    !-- GK2A IR3 (ch.9)

    i_channel = i_channel + 1
    SpcCoeff_wavelen( i_channel ) = SpcCoeff_Def( &
         "GK2A IR3", &
       6.9435713007642033_fp_kind, &
       7.379260510755782900E+03_fp_kind, 2.072096548316442300E+03_fp_kind,                                    &
       2.171564507898664800E-01_fp_kind, 9.991448665749121900E-01_fp_kind,                                          &
       3.331337602932946700E-01_fp_kind, 9.980681851950616900E-01_fp_kind,  2.340611695329752300E-06_fp_kind,      &
       -3.336599637523344800E-01_fp_kind,  1.0019354390242847_fp_kind, -2.346626485658371500E-06_fp_kind )

    !-- GK2A IR4 (ch.10)

    i_channel = i_channel + 1
    SpcCoeff_wavelen( i_channel ) = SpcCoeff_Def( &
         "GK2A IR4", &
       7.3260313560840684_fp_kind, &
       5.643954112075737600E+03_fp_kind, 1.963921450234304500E+03_fp_kind,                                    &
       3.731600717437866300E-02_fp_kind, 9.998464722076462200E-01_fp_kind,                                          &
       6.112692615344893900E-02_fp_kind, 9.996254223039051600E-01_fp_kind,  4.805432690047684100E-07_fp_kind,      &
       -6.114484894154002100E-02_fp_kind,  1.0003747062790485_fp_kind, -4.807646663377694900E-07_fp_kind )

    !-- GK2A IR5 (ch.11)

    i_channel = i_channel + 1
    SpcCoeff_wavelen( i_channel ) = SpcCoeff_Def( &
         "GK2A IR5", &
        8.5883485835352733_fp_kind, &
        2.549053928401534000E+03_fp_kind,       1.675263874697095400E+03_fp_kind,                              &
        6.490647450101244600E-02_fp_kind, 9.996906956030660000E-01_fp_kind,                                    &
        1.401268644532433800E-01_fp_kind,      9.989923832039132600E-01_fp_kind,  1.518070432942246000E-06_fp_kind,      &
        -1.402184215433395500E-01_fp_kind, 1.0010083542788311_fp_kind, -1.519479919625385100E-06_fp_kind )

    !-- GK2A IR6 (ch.12)

    i_channel = i_channel + 1
    SpcCoeff_wavelen( i_channel ) = SpcCoeff_Def( &
         "GK2A IR6", &
        9.6200733880211704_fp_kind, &
        1.445557642881654800E+03_fp_kind,  1.495596711686015900E+03_fp_kind,                                    &
        3.688863334812027300E-02_fp_kind,  9.997964981486259200E-01_fp_kind,                                          &
        1.120006319252027500E-01_fp_kind, 9.990991920064364900E-01_fp_kind,  1.515882917805556200E-06_fp_kind,      &
       -1.120519976218280100E-01_fp_kind,  1.0009012575301879_fp_kind, -1.516808733404820000E-06_fp_kind )

    !-- GK2A IR7 (ch.13)

    i_channel = i_channel + 1
    SpcCoeff_wavelen( i_channel ) = SpcCoeff_Def( &
         "GK2A IR7", &
        1.035658344086081700E+01_fp_kind, &
        9.996426960011384600E+02_fp_kind,  1.389237117381515000E+03_fp_kind,                                    &
       3.136127082189202300E-02_fp_kind, 9.997970822448732900E-01_fp_kind,                                          &
       1.390165431760393100E-01_fp_kind, 9.987976589293754400E-01_fp_kind,  2.172659381520134000E-06_fp_kind,      &
      -1.390819359839989600E-01_fp_kind,  1.001202949635011_fp_kind, -2.173982494463140300E-06_fp_kind )

    !-- GK2A IR8 (ch.14)

    i_channel = i_channel + 1
    SpcCoeff_wavelen( i_channel ) = SpcCoeff_Def( &
         "GK2A IR8", &
        1.122717415477442800E+01_fp_kind, &
        6.676902626675379200E+02_fp_kind,       1.281511262492007500E+03_fp_kind,                              &
     2.244640043064819900E-02_fp_kind, 9.997721481266073100E-01_fp_kind,                                    &
        2.386903693232889900E-01_fp_kind,      9.977646360036879200E-01_fp_kind, 4.364156788957865500E-06_fp_kind,      &
      -2.388133234647361000E-01_fp_kind,     1.0022366291059985_fp_kind, -4.367140957121699500E-06_fp_kind )

    !-- GK2A IR9 (ch.15)

    i_channel = i_channel + 1
    SpcCoeff_wavelen( i_channel ) = SpcCoeff_Def( &
         "GK2A IR9", &
        1.236490937543294100E+01_fp_kind, &
        4.120715691126384900E+02_fp_kind, 1.163595275019866000E+03_fp_kind,                              &
     -4.129780338708610000E-02_fp_kind,  9.999392399122406500E-01_fp_kind,                                   &
       4.270417887649454200E-01_fp_kind, 9.955913849699612500E-01_fp_kind, 9.451858570173908100E-06_fp_kind,      &
     -4.269980541210710600E-01_fp_kind, 1.0044088431875577_fp_kind, -9.453577601214647400E-06_fp_kind )

    !-- GK2A IR10 (ch.16)

    i_channel = i_channel + 1
    SpcCoeff_wavelen( i_channel ) = SpcCoeff_Def( &
         "GK2A IR10", &
        1.327656765696489100E+01_fp_kind, &
        2.887342311056900700E+02_fp_kind,  1.083695010415948400E+03_fp_kind,                                    &
       -2.298360413248732100E-02_fp_kind, 1.0000559492107304_fp_kind,                                     &
       8.514974716659372300E-02_fp_kind, 9.990520876245024800E-01_fp_kind,  2.182307796149972400E-06_fp_kind,      &
      -8.512778286638010600E-02_fp_kind,  1.0009477284102972_fp_kind, -2.181941576447115900E-06_fp_kind )

  end subroutine Init_SpcCoef_wavelen


end module SenPlanck_GK2A
